/* Stats - struct.h - Structure definitions
**
** $Id: struct.h,v 1.24 2001/08/07 15:16:12 wcampbel Exp $
**
*/

#ifndef STRUCT_H
#define STRUCT_H

#include "stats.h"

/* Our data structures are recursive */
typedef struct server_ Server;
typedef struct user_ User;
typedef struct stats_ Stats;
typedef struct tld_ TLD;
typedef struct myuser_ MyUser;
#ifdef IRCD_2_10
typedef struct service_ Service;
#endif

/* The old bit fields are a gcc feature...replace it with bit flags
** and check with & and |
*/
/* Nothing 0x00000001 */
#define FLAG_PRIVMSG	0x00000002
#define FLAG_LAG	0x00000004
#define FLAG_ONLYOPERS	0x00000008
#define FLAG_HTML	0x00000010
#define FLAG_SETOPER	0x00000020
#define FLAG_SETDEAF	0x00000040
#define FLAG_HIDE	0x00000080
#define FLAG_FLOOD	0x00000100
#define FLAG_VHOST	0x00000200

#define UFLAG_FLOODSENT	0x00000001
#define UFLAG_ISOPER	0x00000002
#define UFLAG_ISAWAY	0x00000004

#define MFLAG_ISON	0x00000001

#define CMD_DEBUG       0x00000001   
#define CMD_NOFORK      0x00000002   

#ifdef OPER_MODES
#define OPER_NETADMIN	0x00000001
#define OPER_SVSADMIN	0x00000002
#define OPER_LOCAL	0x00000004
#define OPER_GLOBAL	0x00000008
#define OPER_COADMIN	0x00000010
#define OPER_TECHADMIN	0x00000020
#define OPER_ADMIN	0x00000040
#endif

/* This struct is getting out of hand...
** It needs to be split into at least a netinfo and
** a settings struct.  The remaining things will be 
** in the "catch all" me struct
*/
/* you want it? you got it. -screwedup */

struct netinfo {
	long inb;
	long outb;
        int opers;     
        int maxopers;
        long users;
        long maxusers;
        int servers;
        int maxservers;
        time_t t_maxopers;  
        time_t t_maxusers;  
        time_t t_maxservers;
        unsigned int away; 
        int chan_count;
        int requests;
#ifdef DIE_ON_MASS_COLLIDE
        int collides;
#endif
} netinfo;

struct settings {
        char name[MAXHOST];
        int port;
        unsigned int r_time;
        char uplink[MAXHOST];
        char vhost[MAXHOST];
        char pass[MAXPASS];
        char services_name[MAXHOST];   
        char desc[LINE];
        unsigned int flags;		
        int lag_time;
        char html[BUFSIZE];  
        int html_time;
        char join_chans[BUFSIZE];
        char url[BUFSIZE];
        int flood;
        int flood_time;
        char netname[MAXNICK];  
        char admin_name[LINE];   
        char admin_email[LINE];
} settings;	

struct me {
	time_t t_start;
	Server *s;
	char *chan_list[MAXCHANS];
	unsigned int cmdflags;
} me;

struct StatServ {
	char nick[MAXNICK + 1];
	char user[MAXUSER];
	char host[MAXHOST];
	char rname[LINE];
} StatServ;

struct server_ {
	Server *next, *prev;
	char name[MAXHOST];
	char desc[LINE];
#ifdef TRACK_VERSION
	char version[BUFSIZE];
#endif
	int hops;
#ifdef IRCD_2_10
	int token;
#endif
	long hash;
	Stats *stats;
	unsigned int users;
	int opers;
	time_t connected_since;
	time_t last_announce;
#ifdef LAG_CHECK
	long ping;
	long lowest_ping;
	long highest_ping;
	time_t t_lowest_ping;
	time_t t_highest_ping;
#endif
	Server *uplink;
};

struct user_ {
	User *next, *prev;
	char nick[MAXNICK + 1];
	char *hostname;
	char *username;
	Server *server;
	TLD *tld;
	MyUser *myuser;
	int flood;
	time_t t_flood;
	long hash;
#ifdef OPER_MODES
	unsigned int operflags;
#endif
	unsigned int flags;
};

#ifdef IRCD_2_10
struct service_ {
	Service *next, *prev;
	char nick[MAXNICK + 1];
        char *server;
        char *dist;
	char *gecos;
};
#endif

struct stats_ {
	Stats *next;
	char name[MAXHOST];
	int numsplits;
	long maxusers;
	time_t t_maxusers;
	int maxopers;
	time_t t_maxopers;
	time_t lastseen;
	time_t starttime;
	unsigned int operkills;
	unsigned int serverkills;
};

#ifdef LAG_CHECK
struct ping {
    time_t last_sent;
    long ulag;
} ping;
#endif

struct daily {
	int servers;
	time_t t_servers;
	long users;
	time_t t_users;
	int opers;
	time_t t_opers;
} daily;

struct tld_ {
	TLD *next;
	char tld[MAXTLD];
	char *country;
	int users;
};

struct myuser_ {
	MyUser *next;
	char *username;
	char *password;
	time_t lastseen;
	User *u;
	unsigned int flags;
};

struct MsgTable {
        char* name;
	long counter;
        int (*handler)();
};

#ifdef SPLITSERV
typedef struct spserver_ SpServer;

struct spserver_ {
        char* name;
        SpServer* uplink;
        int numleaf;
        time_t ts;
        SpServer* leaflist[MAXSERVER];
};

struct SplitServ {
        char host[MAXHOST];
        char user[MAXUSER];
        char nick[MAXNICK];
} SplitServ;

struct sp {
        char uplink[MAXHOST];
        char desc[LINE];
        char version[LINE];
        int privmsg : 1;
        int chanmsg : 1;
        int numserver;
        char join_chan[LINE];
        int delay;
        int enable_text : 1;
        int text_time;
        char text_name[BUFSIZE];
} sp;
#endif

struct AdminServ {
        char host[MAXHOST];
        char user[MAXUSER];
        char nick[MAXNICK];
} AdminServ;

#endif
