/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: conf.c,v 1.16 2001/08/06 02:23:39 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

/* Remove CR and LF from the line input.  ASSUME that the line 
** does NOT contain CR or LF in the middle
*/
void strip(char *line)
{
	char *c;

	if ((c = strchr(line, '\n')))	*c = '\0';
	if ((c = strchr(line, '\r')))	*c = '\0';
}

/* Set default parameters.  The settings used are the same as
** what is provided in stats.cfg.dist except for the binary
** ones, which MUST be default 0 in order for the conf load
** to work properly
*/
void InitDefaults()
{
	/* All bit values must be initialized to 0 */
	strcpy(settings.name, "stats.network.tld");
	settings.port = 6666;
	strcpy(settings.uplink, "hub.network.tld");
	strcpy(settings.vhost, "0.0.0.0");
	strcpy(settings.pass, "password");
	strcpy(settings.desc, "Network Statistical Services");
	settings.flags=0x00000000; /* 0 is fine, but be explicit */
	settings.r_time = 10;
	settings.flood = 6;
	settings.flood_time = 10;
	strcpy(StatServ.host, "stats.network.tld");
	strcpy(StatServ.user, "stats");
	strcpy(settings.services_name, "services.network.tld");
	settings.lag_time = 45;
	strcpy(settings.html, "/home/user/public_html/stats.html");
	settings.html_time = 3600;
	strcpy(settings.join_chans, "#statserv,#opers");
	strcpy(settings.url, "http://www.network.tld/stats.html");
	strcpy(StatServ.nick, "StatServ");
	strcpy(settings.netname, "Misconfigured");
	strcpy(settings.admin_name, "Misconfigured Admin (mis-nick)");
	strcpy(settings.admin_email, "admin@misconfigured.net");
	/* The chan_list is derived from the join_chans...don't set it */
#ifdef SPLITSERV
	strcpy(sp.desc, "Split Services");
	sp.delay = 15;
	strcpy(SplitServ.host, "split.network.tld");
	strcpy(SplitServ.user, "split");
	strcpy(SplitServ.nick, "SplitServ");
	sp.privmsg = 0;
	sp.chanmsg = 0;
	sp.enable_text = 0;
	sp.text_time = 1800;
	strcpy(sp.text_name, "map.txt");
	strcpy(sp.join_chan, "#split");
#endif
	strcpy(AdminServ.nick, "AdminServ");
	strcpy(AdminServ.user, "admin");
	strcpy(AdminServ.host, "admin.network.tld");
}

/* Load the conf file 
** Some of these should be modified to allow multiple lines...
** SERVICES_NAME to allow multiple servers to be hidden
** CHANNEL and HTML_LINK as well
*/
void ConfLoad()
{
	FILE *fp;
	char buf[BUFSIZE];
	char *s;

	if ((fp = fopen(CONFIG_NAME, "r")) == NULL) {
		printf("Unable to open configuration file (%s)\n", CONFIG_NAME);
		exit(0);
	}

	while (fgets(buf, BUFSIZE, fp)) {
		if (buf[0] == '#')
			continue;
		s = strtok(buf, " ");
		if (!strcmp(s, "SERVER_NAME")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.name, s, sizeof(settings.name));
			settings.name[MAXHOST - 1] = '\0';
		} else if (!strcmp(s, "SERVER_PORT")) {
			s = strtok(NULL, "\n");
			settings.port = atoi(s);
		} else if (!strcmp(s, "CONNECT_TO")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.uplink, s, sizeof(settings.uplink));
			settings.uplink[MAXHOST - 1] = '\0';
		} else if (!strcmp(s, "CONNECT_PASS")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.pass, s, sizeof(settings.pass));
			settings.pass[MAXPASS - 1] = '\0';
		} else if (!strncmp(s, "USE_VHOST", 9)) {
			settings.flags |= FLAG_VHOST;
		} else if (!strcmp(s, "VHOST")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.vhost, s, sizeof(settings.vhost));
			settings.vhost[MAXHOST - 1] = '\0';
		} else if (!strcmp(s, "SERVER_DESC")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.desc, s, sizeof(settings.desc));
			settings.desc[LINE - 1] = '\0';
		} else if (!strcmp(s, "NETNAME")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.netname, s, sizeof(settings.netname));
			settings.netname[MAXNICK - 1] = '\0';
		} else if (!strcmp(s, "ADMIN_NAME")) {
			s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(settings.admin_name, s, sizeof(settings.admin_name));
			settings.admin_name[LINE - 1] = '\0';
		} else if (!strcmp(s, "ADMIN_EMAIL")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(settings.admin_email, s, sizeof(settings.admin_email));
			settings.admin_email[LINE - 1] = '\0';
		} else if (!strcmp(s, "RECONNECT_TIME")) {
			s = strtok(NULL, "\n");
			settings.r_time = atoi(s);
		} else if (!strcmp(s, "FLOOD")) {
			s = strtok(NULL, "\n");
			settings.flood = atoi(s);
		} else if (!strcmp(s, "FLOOD_TIME")) {
			s = strtok(NULL, "\n");
			settings.flood_time = atoi(s);
		} else if (!strncmp(s, "FLOOD_IGNORE", 12)) {
			settings.flags |= FLAG_FLOOD;
		} else if (!strcmp(s, "LAG_TIME")) {
			s = strtok(NULL, "\n");
			settings.lag_time = atoi(s);
		} else if (!strcmp(s, "STATSERV_HOST")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(StatServ.host, s, sizeof(StatServ.host));
			StatServ.host[MAXHOST - 1] = '\0';
		} else if (!strcmp(s, "STATSERV_USER")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(StatServ.user, s, sizeof(StatServ.user));
			StatServ.user[MAXUSER - 1] = '\0';
		} else if (!strncmp(s, "ENABLE_SPAM", 11)) {
			printf("ENABLE_SPAM is a compile time option.\n");
			printf("Please remove this option from the\n");
			printf("%s file.\nNOT FATAL\n", CONFIG_NAME);
		} else if (!strncmp(s, "ONLYOPERS", 9)) {
			settings.flags |= FLAG_ONLYOPERS;
		} else if (!strcmp(s, "STATSERV_NICK")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(StatServ.nick, s, sizeof(StatServ.nick));
			StatServ.nick[MAXNICK - 1] = '\0';
		} else if (!strncmp(s, "WANT_PRIVMSG", 12)) {
			settings.flags |= FLAG_PRIVMSG;
		} else if (!strcmp(s, "SERVICES_NAME")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.services_name, s, MAXHOST);
			settings.services_name[MAXHOST - 1] = '\0';
		} else if (!strncmp(s, "LAG_NOTICES", 11)) {
			settings.flags |= FLAG_LAG;
		} else if (!strncmp(s, "ENABLE_HTML", 11)) {
			settings.flags |= FLAG_HTML;
		} else if (!strncmp(s, "SET_OPER", 8)) {
			settings.flags |= FLAG_SETOPER;
		} else if (!strncmp(s, "SET_DEAF", 8)) {
			settings.flags |= FLAG_SETDEAF;
		} else if (!strncmp(s, "HIDE_SERVICES", 13)) {
			settings.flags |= FLAG_HIDE;
		} else if (!strcmp(s, "HTML_DUMP")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.html, s, sizeof(settings.html));
			settings.html[BUFSIZE - 1] = '\0';
		} else if (!strcmp(s, "HTML_TIME")) {
			int j;
			s = strtok(NULL, "\n");
			strip(s);
			j = atoi(s);
			settings.html_time = j;
		} else if (!strcmp(s, "IRCD_TYPE")) {
			printf("The IRCD_TYPE Option has been disabled.\n");
			printf("Please remove the IRCD_TYPE from %s\n", 
				CONFIG_NAME);
			printf("and choose the IRCD_type define in config.h\n");
			printf("\nStats terminated.\n");
			exit(1);
		} else if (!strcmp(s, "CHANNELS")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.join_chans, s, sizeof(settings.join_chans));
			settings.join_chans[BUFSIZE - 1] = '\0';
		} else if (!strcmp(s, "URL")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(settings.url, s, sizeof(settings.url));
			settings.url[BUFSIZE - 1] = '\0';
#ifdef SPLITSERV
		} else if (!strcmp(s, "SPLITSERV_DESC")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(sp.desc, s, sizeof(sp.desc));
		} else if (!strcmp(s, "SPLITSERV_DELAY")) {
			int j;
                        s = strtok(NULL, "\n");
                        strip(s);
                        j = atoi(s);
                        sp.delay = j;
		} else if (!strcmp(s, "SPLITSERV_HOST")) {
			s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(SplitServ.host, s, sizeof(SplitServ.host));
                } else if (!strcmp(s, "SPLITSERV_USER")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(SplitServ.user, s, sizeof(SplitServ.user));
                } else if (!strcmp(s, "SPLITSERV_NICK")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(SplitServ.nick, s, sizeof(SplitServ.nick));
                } else if (!strncmp(s, "SPLITSERV_PRIVMSG", 17)) {
                        sp.privmsg = 1;
                } else if (!strncmp(s, "SPLITSERV_CHANMSG", 17)) {
                        sp.chanmsg = 1;
                } else if (!strncmp(s, "SPLITSERV_ENABLE_TEXT", 21)) {
                        sp.enable_text = 1;
                } else if (!strcmp(s, "SPLITSERV_TEXT_TIME")) {
                        int j;
                        s = strtok(NULL, "\n");
                        strip(s);
                        j = atoi(s);
                        sp.text_time = j;
                } else if (!strcmp(s, "SPLITSERV_TEXT_NAME")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(sp.text_name, s, sizeof(sp.text_name));
                } else if (!strcmp(s, "SPLITSERV_CHAN")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(sp.join_chan, s, sizeof(sp.join_chan));
#endif
                } else if (!strcmp(s, "ADMINSERV_HOST")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(AdminServ.host, s, sizeof(AdminServ.host));
                } else if (!strcmp(s, "ADMINSERV_USER")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(AdminServ.user, s, sizeof(AdminServ.user));
                } else if (!strcmp(s, "ADMINSERV_NICK")) {
                        s = strtok(NULL, "\n");
                        strip(s);
                        memcpy(AdminServ.nick, s, sizeof(AdminServ.nick));
		}
	}
	fclose(fp);
}

int GetChanList()
{
	char *s;
	int count = 0;
	for (s = strtok(settings.join_chans, ",");
	     s;
	     s = strtok(NULL, ",")) {
		if (count == MAXCHANS) {
			log("Maximum number of channels join'ed");
			return count;
		}
		/* I know the ++ can be inside the brackets...it's just less
		** readable that way
		*/
		if (*s == ' ')
			s++;
		me.chan_list[count] = strdup(s);
		count++;
	}

	return count;
}

void rehash()
{
	/* still nothing, yet */

	/* I need to add in a limited conf reloading - mark flags
	** for what has been changed, send notices etc.  Some thing
	** won't be able to dynamically change, either do a disconnect
	** and reconnect or do a restart when needed... or demand the 
	** admin do it...bloody complicated
	*/

	/* Add an option to reload the user file as well - maybe in
	** a seperate function
	*/

	/* Free and reload the TLD file too */

	/* I should probably have dynamic settings too, and will need
	** a file to store dynamic settings in, read after the main 
	** conf file... gets confusing...bah
	*/
}
